@@ -7,7 +7,7 @@ package AnyEvent::Redis::RipeRedis;
 
 use base qw( Exporter );
 
-our $VERSION = '1.45_02';
+our $VERSION = '1.45_03';
 
 use AnyEvent;
 use AnyEvent::Handle;
@@ -92,7 +92,8 @@ my %SUBUNSUB_CMDS = (
   unsubscribe  => 1,
   punsubscribe => 1,
 );
-my %NEED_SPECPROC = (
+
+my %NEED_POSTPROCESS = (
   %SUBUNSUB_CMDS,
   info   => 1,
   select => 1,
@@ -116,6 +117,7 @@ my %ERR_PREFS_MAP = (
   NOAUTH     => E_NO_AUTH,
   WRONGTYPE  => E_WRONG_TYPE,
   NOREPLICAS => E_NO_REPLICAS,
+  BUSYKEY    => E_BUSY_KEY,
 );
 
 my %EVAL_CACHE;
@@ -159,7 +161,7 @@ sub new {
   $self->{_input_queue}    = [];
   $self->{_tmp_queue}      = [];
   $self->{_process_queue}  = [];
-  $self->{_multi_lock}     = 0;
+  $self->{_txn_lock}       = 0;
   $self->{_subs}           = {};
   $self->{_subs_num}       = 0;
 
@@ -175,7 +177,7 @@ sub multi {
   my $self = shift;
   my $cmd  = $self->_prepare_cmd( 'multi', [ @_ ] );
 
-  $self->{_multi_lock} = 1;
+  $self->{_txn_lock} = 1;
   $self->_execute_cmd( $cmd );
 
   return;
@@ -186,7 +188,7 @@ sub exec {
   my $self = shift;
   my $cmd  = $self->_prepare_cmd( 'exec', [ @_ ] );
 
-  $self->{_multi_lock} = 0;
+  $self->{_txn_lock} = 0;
   $self->_execute_cmd( $cmd );
 
   return;
@@ -277,7 +279,34 @@ sub selected_database {
       my $self = shift;
       my $cmd  = $self->_prepare_cmd( $kwd, [ @_ ] );
 
-      $self->_subunsub( $cmd );
+      if ( exists $SUB_CMDS{ $cmd->{kwd} } && !defined $cmd->{on_message} ) {
+        croak "'on_message' callback must be specified";
+      }
+
+      if ( $self->{_txn_lock} ) {
+        AE::postpone(
+          sub {
+            $self->_process_cmd_error( $cmd,
+                "Command '$cmd->{kwd}' not allowed after 'multi' command."
+                    . ' First, the transaction must be finalized.',
+                E_OPRN_ERROR );
+          }
+        );
+
+        return;
+      }
+
+      $cmd->{replies_cnt} = scalar @{ $cmd->{args} };
+
+      if ( defined $cmd->{on_done} ) {
+        my $on_done = $cmd->{on_done};
+
+        $cmd->{on_done} = sub {
+          $on_done->( @{ $_[0] } );
+        }
+      }
+
+      $self->_execute_cmd( $cmd );
 
       return;
     },
@@ -403,7 +432,7 @@ sub _get_on_rtimeout {
   weaken( $self );
 
   return sub {
-    if ( @{$self->{_process_queue}} ) {
+    if ( @{ $self->{_process_queue} } ) {
       $self->_disconnect( 'Read timed out.', E_READ_TIMEDOUT );
     }
     else {
@@ -494,9 +523,9 @@ sub _get_on_read {
           elsif ( $type eq '*' ) {
             if ( $data > 0 ) {
               push( @bufs,
-                { data        => [],
-                  err_code    => undef,
-                  chunks_left => $data,
+                { data       => [],
+                  err_code   => undef,
+                  chunks_cnt => $data,
                 }
               );
               $bufs_num++;
@@ -513,8 +542,8 @@ sub _get_on_read {
           elsif ( $type eq '-' ) {
             $err_code = E_OPRN_ERROR;
             if ( $data =~ m/^([A-Z]{3,}) / ) {
-              if ( exists $ERR_PREFS_MAP{ $1 } ) {
-                $err_code = $ERR_PREFS_MAP{ $1 };
+              if ( exists $ERR_PREFS_MAP{$1} ) {
+                $err_code = $ERR_PREFS_MAP{$1};
               }
             }
           }
@@ -535,8 +564,8 @@ sub _get_on_read {
           }
           $curr_buf->{err_code} = E_OPRN_ERROR;
         }
-        push( @{$curr_buf->{data}}, $data );
-        if ( --$curr_buf->{chunks_left} > 0 ) {
+        push( @{ $curr_buf->{data} }, $data );
+        if ( --$curr_buf->{chunks_cnt} > 0 ) {
           next MAIN;
         }
 
@@ -580,43 +609,6 @@ sub _prepare_cmd {
   return $cmd;
 }
 
-####
-sub _subunsub {
-  my $self = shift;
-  my $cmd  = shift;
-
-  if ( exists $SUB_CMDS{ $cmd->{kwd} } && !defined $cmd->{on_message} ) {
-    croak "'on_message' callback must be specified";
-  }
-
-  if ( $self->{_multi_lock} ) {
-    AE::postpone(
-      sub {
-        $self->_process_cmd_error( $cmd,
-            "Command '$cmd->{kwd}' not allowed after 'multi' command."
-                . ' First, the transaction must be finalized.',
-            E_OPRN_ERROR );
-      }
-    );
-
-    return;
-  }
-
-  $cmd->{repls_left} = scalar @{ $cmd->{args} };
-
-  if ( defined $cmd->{on_done} ) {
-    my $on_done = $cmd->{on_done};
-
-    $cmd->{on_done} = sub {
-      $on_done->( @{ $_[0] } );
-    }
-  }
-
-  $self->_execute_cmd( $cmd );
-
-  return;
-}
-
 ####
 sub _execute_cmd {
   my $self = shift;
@@ -653,7 +645,7 @@ sub _execute_cmd {
       return;
     }
 
-    push( @{$self->{_input_queue}}, $cmd );
+    push( @{ $self->{_input_queue} }, $cmd );
 
     return;
   }
@@ -669,7 +661,7 @@ sub _push_write {
   my $cmd  = shift;
 
   my $cmd_str = '';
-  foreach my $token ( $cmd->{kwd}, @{$cmd->{args}} ) {
+  foreach my $token ( $cmd->{kwd}, @{ $cmd->{args} } ) {
     unless ( defined $token ) {
       $token = '';
     }
@@ -678,14 +670,14 @@ sub _push_write {
     }
     $cmd_str .= '$' . length( $token ) . EOL . $token . EOL;
   }
-  $cmd_str = '*' . ( scalar( @{$cmd->{args}} ) + 1 ) . EOL . $cmd_str;
+  $cmd_str = '*' . ( scalar( @{ $cmd->{args} } ) + 1 ) . EOL . $cmd_str;
 
   my $handle = $self->{_handle};
-  if ( defined $self->{read_timeout} && !@{$self->{_process_queue}} ) {
+  if ( defined $self->{read_timeout} && !@{ $self->{_process_queue} } ) {
     $handle->rtimeout_reset();
     $handle->rtimeout( $self->{read_timeout} );
   }
-  push( @{$self->{_process_queue}}, $cmd );
+  push( @{ $self->{_process_queue} }, $cmd );
 
   $handle->push_write( $cmd_str );
 
@@ -739,7 +731,7 @@ sub _select_db {
       args => [ $self->{database} ],
 
       on_done => sub {
-        $self->{_select_db_st} = S_IS_DONE;
+        $self->{_select_db_st}   = S_IS_DONE;
         $self->{_ready_to_write} = 1;
         $self->_flush_input_queue();
       },
@@ -775,10 +767,10 @@ sub _process_reply {
   my $err_code = shift;
 
   if ( defined $err_code ) {
-    my $cmd = shift @{$self->{_process_queue}};
+    my $cmd = shift @{ $self->{_process_queue} };
 
     unless ( defined $cmd ) {
-      $self->_disconnect( "Don't known how process error."
+      $self->_disconnect( "Don't known how process error message."
           . ' Processing queue is empty.', E_UNEXPECTED_DATA );
 
       return;
@@ -810,8 +802,8 @@ sub _process_reply {
       return;
     }
 
-    if ( !defined $cmd->{repls_left} || --$cmd->{repls_left} <= 0 ) {
-      shift @{$self->{_process_queue}};
+    if ( !defined $cmd->{replies_cnt} || --$cmd->{replies_cnt} <= 0 ) {
+      shift @{ $self->{_process_queue} };
     }
     $self->_process_cmd_success( $cmd, $data );
   }
@@ -852,7 +844,7 @@ sub _process_cmd_success {
   my $cmd  = shift;
   my $data = shift;
 
-  if ( exists $NEED_SPECPROC{ $cmd->{kwd} } ) {
+  if ( exists $NEED_POSTPROCESS{ $cmd->{kwd} } ) {
     my $kwd = $cmd->{kwd};
 
     if ( exists $SUBUNSUB_CMDS{$kwd} ) {
@@ -910,7 +902,7 @@ sub _disconnect {
   $self->{_auth_st}        = S_NEED_PERFORM;
   $self->{_select_db_st}   = S_NEED_PERFORM;
   $self->{_ready_to_write} = 0;
-  $self->{_multi_lock}     = 0;
+  $self->{_txn_lock}       = 0;
 
   $self->_abort_all( @_ );
 
@@ -928,9 +920,9 @@ sub _abort_all {
   my $err_code = shift;
 
   my @unfin_cmds = (
-    @{$self->{_process_queue}},
-    @{$self->{_tmp_queue}},
-    @{$self->{_input_queue}},
+    @{ $self->{_process_queue} },
+    @{ $self->{_tmp_queue} },
+    @{ $self->{_input_queue} },
   );
 
   $self->{_input_queue}   = [];
@@ -990,9 +982,9 @@ sub DESTROY {
 
   if ( defined $self->{_handle} ) {
     my @unfin_cmds = (
-      @{$self->{_process_queue}},
-      @{$self->{_tmp_queue}},
-      @{$self->{_input_queue}},
+      @{ $self->{_process_queue} },
+      @{ $self->{_tmp_queue} },
+      @{ $self->{_input_queue} },
     );
 
     foreach my $cmd ( @unfin_cmds ) {
